﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Globalization;
using System.Linq;
using System.Text;

namespace HeuristicLab.Problems.VehicleRouting.ProblemInstances {
  public class CVRPStopInsertionInfo : StopInsertionInfo {
    private double spareCapacity;

    public double SpareCapacity {
      get { return spareCapacity; }
    }

    public CVRPStopInsertionInfo(int start, int end, double distance, double spareCapacity)
      : base(start, end, distance) {
      this.spareCapacity = spareCapacity;
    }

    protected override void GetStopInsertionReport(StringBuilder builder) {
      base.GetStopInsertionReport(builder);
      builder.Append("Capacity left:\t");
      builder.AppendLine(SpareCapacity.ToString(CultureInfo.CurrentCulture));
    }
  }

  public class CVRPTourInsertionInfo : TourInsertionInfo {
    private double capacity;

    public virtual double SpareCapacity {
      get { return stopInsertionInfos.Count == 0 ? capacity : ((CVRPStopInsertionInfo)stopInsertionInfos.Last()).SpareCapacity; }
    }
    
    public CVRPTourInsertionInfo(int vehicle, double capacity)
      : base(vehicle) {
      this.capacity = capacity;
    }

    protected override void GetTourInsertionReport(StringBuilder builder) {
      base.GetTourInsertionReport(builder);
      builder.Append("Capacity left:\t");
      builder.AppendLine(SpareCapacity.ToString(CultureInfo.CurrentCulture));
    }
  }

  public class CVRPEvaluation : VRPEvaluation {
    public double Overload { get; set; }

    protected override void GetEvaluationReport(StringBuilder builder) {
      base.GetEvaluationReport(builder);
      builder.Append("Overload:\t");
      builder.AppendLine(Overload.ToString(CultureInfo.CurrentCulture));
    }
  }
}
